@@ -1,60 +1,12 @@
 """
-Ad-hoc tools for drawing Python object reference graphs with graphviz.
+Tools for drawing Python object reference graphs with graphviz.
 
-This module is more useful as a repository of sample code and ideas, than
-as a finished product.  For documentation and background, read
-
-  http://mg.pov.lt/blog/hunting-python-memleaks.html
-  http://mg.pov.lt/blog/python-object-graphs.html
-  http://mg.pov.lt/blog/object-graphs-with-graphviz.html
-
-in that order.  Then use pydoc to read the docstrings, as there were
-improvements made since those blog posts.
+You can find documentation online at http://mg.pov.lt/objgraph/
 
 Copyright (c) 2008-2010 Marius Gedminas <marius@pov.lt>
+Copyright (c) 2010 Stefano Rivera <stefano@rivera.za.net>
 
 Released under the MIT licence.
-
-
-Changes
-=======
-
-1.3 (2010-07-13)
-----------------
-
-Highlight objects with a __del__ method.
-
-Fixes LP#483411, LP#514422, 
-
-show_refs, show_backrefs don't create files in the current working
-directory any more.  Instead they accept a filename argument.  It can be a
-.dot file or a .png file.  If None or not specified, those functions will try
-to spawn xdot as before.  Fixes LP#514422.
-
-New extra_info argument to graph-generating functions (patch by Thouis Jones,
-LP#558914).
-
-setup.py should work with distutils now (LP#604430, thanks to Randy Heydon).
-
-
-1.2 (2009-03-25)
-----------------
-
-Project website, public source repository, uploaded to PyPI.
-
-No code changes.
-
-
-1.1dev (2008-09-05)
--------------------
-
-New function: show_refs() for showing forward references.
-
-New functions: typestats() and show_most_common_types().
-
-Object boxes are less crammed with useless information (such as IDs).
-
-Spawns xdot if it is available.
 """
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -77,18 +29,32 @@ Spawns xdot if it is available.
 __author__ = "Marius Gedminas (marius@gedmin.as)"
 __copyright__ = "Copyright (c) 2008-2010 Marius Gedminas"
 __license__ = "MIT"
-__version__ = "1.3"
-__date__ = "2010-07-13"
+__version__ = "1.6.0"
+__date__ = "2010-12-18"
 
 
 import gc
 import inspect
 import types
-import weakref
 import operator
 import os
 import subprocess
-from tempfile import NamedTemporaryFile
+import tempfile
+import sys
+import itertools
+
+
+try:
+    basestring
+except NameError:
+    # Python 3.x compatibility
+    basestring = str
+
+try:
+    iteritems = dict.iteritems
+except AttributeError:
+    # Python 3.x compatibility
+    iteritems = dict.items
 
 
 def count(typename):
@@ -113,6 +79,13 @@ def typestats():
 
     Note that classes with the same name but defined in different modules
     will be lumped together.
+
+    Example:
+
+        >>> typestats()
+        {'list': 12041, 'tuple': 10245, ...}
+
+    .. versionadded:: 1.1
     """
     stats = {}
     for o in gc.get_objects():
@@ -121,21 +94,90 @@ def typestats():
     return stats
 
 
-def show_most_common_types(limit=10):
+def most_common_types(limit=10):
     """Count the names of types with the most instances.
 
-    Note that the GC does not track simple objects like int or str.
+    Returns a list of (type_name, count), sorted most-frequent-first.
 
-    Note that classes with the same name but defined in different modules
-    will be lumped together.
+    Limits the return value to at most ``limit`` items.  You may set ``limit``
+    to None to avoid that.
+
+    The caveats documented in :func:`typestats` apply.
+
+    Example:
+
+        >>> most_common_types(limit=2)
+        [('list', 12041), ('tuple', 10245)]
+
+    .. versionadded:: 1.4
     """
     stats = sorted(typestats().items(), key=operator.itemgetter(1),
                    reverse=True)
     if limit:
         stats = stats[:limit]
+    return stats
+
+
+def show_most_common_types(limit=10):
+    """Print the table of types of most common instances
+
+    The caveats documented in :func:`typestats` apply.
+
+    Example:
+
+        >>> show_most_common_types(limit=5)
+        tuple                      8959
+        function                   2442
+        wrapper_descriptor         1048
+        dict                       953
+        builtin_function_or_method 800
+
+    .. versionadded:: 1.1
+    """
+    stats = most_common_types(limit)
     width = max(len(name) for name, count in stats)
-    for name, count in stats[:limit]:
-        print name.ljust(width), count
+    for name, count in stats:
+        print('%-*s %i' % (width, name, count))
+
+
+def show_growth(limit=10, peak_stats={}):
+    """Show the increase in peak object counts since last call.
+
+    Limits the output to ``limit`` largest deltas.  You may set ``limit`` to
+    None to see all of them.
+
+    Uses and updates ``peak_stats``, a dictionary from type names to previously
+    seen peak object counts.  Usually you don't need to pay attention to this
+    argument.
+
+    The caveats documented in :func:`typestats` apply.
+
+    Example:
+
+        >>> objgraph.show_growth()
+        wrapper_descriptor       970       +14
+        tuple                  12282       +10
+        dict                    1922        +7
+        ...
+
+    .. versionadded:: 1.5
+    """
+    gc.collect()
+    stats = typestats()
+    deltas = {}
+    for name, count in iteritems(stats):
+        old_count = peak_stats.get(name, 0)
+        if count > old_count:
+            deltas[name] = count - old_count
+            peak_stats[name] = count
+    deltas = sorted(deltas.items(), key=operator.itemgetter(1),
+                    reverse=True)
+    if limit:
+        deltas = deltas[:limit]
+    if deltas:
+        width = max(len(name) for name, count in deltas)
+        for name, delta in deltas:
+            print('%-*s%9d %+9d' % (width, name, stats[name], delta))
 
 
 def by_type(typename):
@@ -153,7 +195,7 @@ def by_type(typename):
 
 def at(addr):
     """Return an object at a given memory address.
-    
+
     The reverse of id(obj):
 
         >>> at(id(obj)) is obj
@@ -173,6 +215,8 @@ def find_backref_chain(obj, predicate, max_depth=20, extra_ignore=()):
 
     The start of the chain will be some object that matches your predicate.
 
+    ``predicate`` is a function taking one argument and returning a boolean.
+
     ``max_depth`` limits the search depth.
 
     ``extra_ignore`` can be a list of object IDs to exclude those objects from
@@ -183,7 +227,11 @@ def find_backref_chain(obj, predicate, max_depth=20, extra_ignore=()):
         >>> find_backref_chain(obj, inspect.ismodule)
         [<module ...>, ..., obj]
 
-    Returns None if such a chain could not be found.
+    Returns ``[obj]`` if such a chain could not be found.
+
+    .. versionchanged:: 1.5
+       Returns ``obj`` instead of ``None`` when a chain could not be found.
+
     """
     queue = [obj]
     depth = {id(obj): 0}
@@ -194,6 +242,7 @@ def find_backref_chain(obj, predicate, max_depth=20, extra_ignore=()):
     ignore.add(id(depth))
     ignore.add(id(parent))
     ignore.add(id(ignore))
+    ignore.add(id(sys._getframe()))  # this function
     gc.collect()
     while queue:
         target = queue.pop(0)
@@ -208,17 +257,18 @@ def find_backref_chain(obj, predicate, max_depth=20, extra_ignore=()):
             referrers = gc.get_referrers(target)
             ignore.add(id(referrers))
             for source in referrers:
-                if inspect.isframe(source) or id(source) in ignore:
+                if id(source) in ignore:
                     continue
                 if id(source) not in depth:
                     depth[id(source)] = tdepth + 1
                     parent[id(source)] = target
                     queue.append(source)
-    return None # not found
+    return [obj] # not found
 
 
 def show_backrefs(objs, max_depth=3, extra_ignore=(), filter=None, too_many=10,
-                  highlight=None, filename=None, extra_info=(lambda _: '')):
+                  highlight=None, filename=None, extra_info=None,
+                  refcounts=False):
     """Generate an object reference graph ending at ``objs``
 
     The graph will show you what objects refer to ``objs``, directly and
@@ -227,8 +277,11 @@ def show_backrefs(objs, max_depth=3, extra_ignore=(), filter=None, too_many=10,
     ``objs`` can be a single object, or it can be a list of objects.  If
     unsure, wrap the single object in a new list.
 
-    Produces a Graphviz .dot file and spawns a viewer (xdot) if one is
-    installed, otherwise converts the graph to a .png image.
+    ``filename`` if specified, can be the name of a .dot or a .png file,
+    indicating the desired output format.  If not specified, ``show_backrefs``
+    will try to produce a .dot file and spawn a viewer (xdot).  If xdot is
+    not available, ``show_backrefs`` will convert the .dot file to a .png
+    and print its name.
 
     Use ``max_depth`` and ``too_many`` to limit the depth and breadth of the
     graph.
@@ -238,8 +291,12 @@ def show_backrefs(objs, max_depth=3, extra_ignore=(), filter=None, too_many=10,
 
     Use ``highlight`` (a predicate) to highlight certain graph nodes in blue.
 
-    Use ``extra_info`` (a function returning a string) to report extra
-    information for objects.
+    Use ``extra_info`` (a function taking one argument and returning a
+    string) to report extra information for objects.
+
+    Specify ``refcounts=True`` if you want to see reference counts.
+    These will mostly match the number of arrows pointing to an object,
+    but can be different for various reasons.
 
     Examples:
 
@@ -250,15 +307,22 @@ def show_backrefs(objs, max_depth=3, extra_ignore=(), filter=None, too_many=10,
         >>> show_backrefs(obj, highlight=inspect.isclass)
         >>> show_backrefs(obj, extra_ignore=[id(locals())])
 
+    .. versionchanged:: 1.3
+       New parameters: ``filename``, ``extra_info``.
+
+    .. versionchanged:: 1.5
+       New parameter: ``refcounts``.
+
     """
     show_graph(objs, max_depth=max_depth, extra_ignore=extra_ignore,
                filter=filter, too_many=too_many, highlight=highlight,
                edge_func=gc.get_referrers, swap_source_target=False,
-               filename=filename, extra_info=extra_info)
+               filename=filename, extra_info=extra_info, refcounts=refcounts)
 
 
 def show_refs(objs, max_depth=3, extra_ignore=(), filter=None, too_many=10,
-              highlight=None, filename=None, extra_info=(lambda _: '')):
+              highlight=None, filename=None, extra_info=None,
+              refcounts=False):
     """Generate an object reference graph starting at ``objs``
 
     The graph will show you what objects are reachable from ``objs``, directly
@@ -267,8 +331,11 @@ def show_refs(objs, max_depth=3, extra_ignore=(), filter=None, too_many=10,
     ``objs`` can be a single object, or it can be a list of objects.  If
     unsure, wrap the single object in a new list.
 
-    Produces a Graphviz .dot file and spawns a viewer (xdot) if one is
-    installed, otherwise converts the graph to a .png image.
+    ``filename`` if specified, can be the name of a .dot or a .png file,
+    indicating the desired output format.  If not specified, ``show_refs``
+    will try to produce a .dot file and spawn a viewer (xdot).  If xdot is
+    not available, ``show_refs`` will convert the .dot file to a .png
+    and print its name.
 
     Use ``max_depth`` and ``too_many`` to limit the depth and breadth of the
     graph.
@@ -281,6 +348,8 @@ def show_refs(objs, max_depth=3, extra_ignore=(), filter=None, too_many=10,
     Use ``extra_info`` (a function returning a string) to report extra
     information for objects.
 
+    Specify ``refcounts=True`` if you want to see reference counts.
+
     Examples:
 
         >>> show_refs(obj)
@@ -290,11 +359,39 @@ def show_refs(objs, max_depth=3, extra_ignore=(), filter=None, too_many=10,
         >>> show_refs(obj, highlight=inspect.isclass)
         >>> show_refs(obj, extra_ignore=[id(locals())])
 
+    .. versionadded:: 1.1
+
+    .. versionchanged:: 1.3
+       New parameters: ``filename``, ``extra_info``.
+
+    .. versionchanged:: 1.5
+       New parameter: ``refcounts``.
+       Follows references from module objects instead of stopping.
+
     """
     show_graph(objs, max_depth=max_depth, extra_ignore=extra_ignore,
                filter=filter, too_many=too_many, highlight=highlight,
                edge_func=gc.get_referents, swap_source_target=True,
-               filename=filename, extra_info=extra_info)
+               filename=filename, extra_info=extra_info, refcounts=refcounts)
+
+
+def show_chain(*chains, **kw):
+    """Show a chain (or several chains) of object references.
+
+    Useful in combination with :func:`find_backref_chain`, e.g.
+
+        >>> show_chain(find_backref_chain(obj, inspect.ismodule))
+
+    You can specify ``highlight``, ``extra_info`` or ``filename`` arguments
+    like for :func:`show_backrefs`.
+
+    .. versionadded:: 1.5
+    """
+    chains = [chain for chain in chains if chain] # remove empty ones
+    def in_chains(x, ids=set(map(id, itertools.chain(*chains)))):
+        return id(x) in ids
+    show_backrefs([chain[-1] for chain in chains], max(map(len, chains)) - 1,
+                  filter=in_chains, **kw)
 
 #
 # Internal helpers
@@ -302,17 +399,18 @@ def show_refs(objs, max_depth=3, extra_ignore=(), filter=None, too_many=10,
 
 def show_graph(objs, edge_func, swap_source_target,
                max_depth=3, extra_ignore=(), filter=None, too_many=10,
-               highlight=None, filename=None, extra_info=(lambda _: '')):
+               highlight=None, filename=None, extra_info=None,
+               refcounts=False):
     if not isinstance(objs, (list, tuple)):
         objs = [objs]
     if filename and filename.endswith('.dot'):
-        f = file(filename, 'w')
+        f = open(filename, 'w')
         dot_filename = filename
     else:
-        f = NamedTemporaryFile('w', suffix='.dot', delete=False)
-        dot_filename = f.name
-    print >> f, 'digraph ObjectGraph {'
-    print >> f, '  node[shape=box, style=filled, fillcolor=white];'
+        fd, dot_filename = tempfile.mkstemp('.dot', text=True)
+        f = os.fdopen(fd, "w")
+    f.write('digraph ObjectGraph {\n'
+            '  node[shape=box, style=filled, fillcolor=white];\n')
     queue = []
     depth = {}
     ignore = set(extra_ignore)
@@ -321,17 +419,20 @@ def show_graph(objs, edge_func, swap_source_target,
     ignore.add(id(queue))
     ignore.add(id(depth))
     ignore.add(id(ignore))
+    ignore.add(id(sys._getframe()))  # this function
+    ignore.add(id(sys._getframe(1))) # show_refs/show_backrefs, most likely
     for obj in objs:
-        print >> f, '  %s[fontcolor=red];' % (obj_node_id(obj))
+        f.write('  %s[fontcolor=red];\n' % (obj_node_id(obj)))
         depth[id(obj)] = 0
         queue.append(obj)
+        del obj
     gc.collect()
     nodes = 0
     while queue:
         nodes += 1
         target = queue.pop(0)
         tdepth = depth[id(target)]
-        print >> f, '  %s[label="%s"];' % (obj_node_id(target), obj_label(target, tdepth, extra_info))
+        f.write('  %s[label="%s"];\n' % (obj_node_id(target), obj_label(target, extra_info, refcounts)))
         h, s, v = gradient((0, 0, 1), (0, 0, .3), tdepth, max_depth)
         if inspect.ismodule(target):
             h = .3
@@ -340,74 +441,101 @@ def show_graph(objs, edge_func, swap_source_target,
             h = .6
             s = .6
             v = 0.5 + v * 0.5
-        print >> f, '  %s[fillcolor="%g,%g,%g"];' % (obj_node_id(target), h, s, v)
+        f.write('  %s[fillcolor="%g,%g,%g"];\n' % (obj_node_id(target), h, s, v))
         if v < 0.5:
-            print >> f, '  %s[fontcolor=white];' % (obj_node_id(target))
-        if hasattr(target, '__del__'):
-            print >> f, "  %s->%s_has_a_del[color=red,style=dotted,len=0.25,weight=10];" % (obj_node_id(target), obj_node_id(target))
-            print >> f, '  %s_has_a_del[label="__del__",shape=doublecircle,height=0.25,color=red,fillcolor="0,.5,1",fontsize=6];' % (obj_node_id(target))
-        if inspect.ismodule(target) or tdepth >= max_depth:
+            f.write('  %s[fontcolor=white];\n' % (obj_node_id(target)))
+        if hasattr(getattr(target, '__class__', None), '__del__'):
+            f.write("  %s->%s_has_a_del[color=red,style=dotted,len=0.25,weight=10];\n" % (obj_node_id(target), obj_node_id(target)))
+            f.write('  %s_has_a_del[label="__del__",shape=doublecircle,height=0.25,color=red,fillcolor="0,.5,1",fontsize=6];\n' % (obj_node_id(target)))
+        if tdepth >= max_depth:
+            continue
+        if inspect.ismodule(target) and not swap_source_target:
+            # For show_backrefs(), it makes sense to stop when reaching a
+            # module because you'll end up in sys.modules and explode the
+            # graph with useless clutter.  For show_refs(), it makes sense
+            # to continue.
             continue
         neighbours = edge_func(target)
         ignore.add(id(neighbours))
         n = 0
+        skipped = 0
         for source in neighbours:
-            if inspect.isframe(source) or id(source) in ignore:
+            if id(source) in ignore:
                 continue
             if filter and not filter(source):
                 continue
+            if n >= too_many:
+                skipped += 1
+                continue
             if swap_source_target:
                 srcnode, tgtnode = target, source
             else:
                 srcnode, tgtnode = source, target
             elabel = edge_label(srcnode, tgtnode)
-            print >> f, '  %s -> %s%s;' % (obj_node_id(srcnode), obj_node_id(tgtnode), elabel)
+            f.write('  %s -> %s%s;\n' % (obj_node_id(srcnode), obj_node_id(tgtnode), elabel))
             if id(source) not in depth:
                 depth[id(source)] = tdepth + 1
                 queue.append(source)
             n += 1
-            if n >= too_many:
-                print >> f, '  %s[color=red];' % obj_node_id(target)
-                break
-    print >> f, "}"
+            del source
+        del neighbours
+        if skipped > 0:
+            h, s, v = gradient((0, 1, 1), (0, 1, .3), tdepth + 1, max_depth)
+            if swap_source_target:
+                label = "%d more references" % skipped
+                edge = "%s->too_many_%s" % (obj_node_id(target), obj_node_id(target))
+            else:
+                label = "%d more backreferences" % skipped
+                edge = "too_many_%s->%s" % (obj_node_id(target), obj_node_id(target))
+            f.write('  %s[color=red,style=dotted,len=0.25,weight=10];\n' % edge)
+            f.write('  too_many_%s[label="%s",shape=box,height=0.25,color=red,fillcolor="%g,%g,%g",fontsize=6];\n' % (obj_node_id(target), label, h, s, v))
+            f.write('  too_many_%s[fontcolor=white];\n' % (obj_node_id(target)))
+    f.write("}\n")
     f.close()
-    print "Graph written to %s (%d nodes)" % (dot_filename, nodes)
-    if filename is None and program_in_path('xdot'):
-        print "Spawning graph viewer (xdot)"
-        subprocess.Popen(['xdot', dot_filename])
+    print("Graph written to %s (%d nodes)" % (dot_filename, nodes))
+    if not filename and program_in_path('xdot'):
+        print("Spawning graph viewer (xdot)")
+        subprocess.Popen(['xdot', dot_filename], close_fds=True)
     elif program_in_path('dot'):
-        if filename is None:
-            print "Graph viewer (xdot) not found, generating a png instead"
+        if not filename:
+            print("Graph viewer (xdot) not found, generating a png instead")
         if filename and filename.endswith('.png'):
-            f = file(filename, 'wb')
+            f = open(filename, 'wb')
             png_filename = filename
         else:
-            if filename is not None:
-                print "Unrecognized file type (%s)" % filename
-            f = NamedTemporaryFile('wb', suffix='.png', delete=False)
-            png_filename = f.name
+            if filename:
+                print("Unrecognized file type (%s)" % filename)
+            fd, png_filename = tempfile.mkstemp('.png', text=False)
+            f = os.fdopen(fd, "wb")
         dot = subprocess.Popen(['dot', '-Tpng', dot_filename],
-                               stdout=f)
+                               stdout=f, close_fds=False)
         dot.wait()
         f.close()
-        print "Image generated as %s" % png_filename
+        print("Image generated as %s" % png_filename)
     else:
-        if filename is None:
-            print "Graph viewer (xdot) and image renderer (dot) not found, not doing anything else"
+        if filename:
+            print("Graph viewer (xdot) and image renderer (dot) not found, not doing anything else")
         else:
-            print "Unrecognized file type (%s), not doing anything else" % filename
+            print("Unrecognized file type (%s), not doing anything else" % filename)
 
 
 def obj_node_id(obj):
-    if isinstance(obj, weakref.ref):
-        return 'all_weakrefs_are_one'
     return ('o%d' % id(obj)).replace('-', '_')
 
 
-def obj_label(obj, depth, extra_info):
-    return quote(type(obj).__name__ + ':\n' +
-                 safe_repr(obj) + '\n' +
-                 extra_info(obj))
+def obj_label(obj, extra_info=None, refcounts=False):
+    label = [type(obj).__name__]
+    if refcounts:
+        label[0] += ' [%d]' % (sys.getrefcount(obj) - 4)
+        # Why -4?  To ignore the references coming from
+        #   obj_label's frame (obj)
+        #   show_graph's frame (target variable)
+        #   sys.getrefcount()'s argument
+        #   something else that doesn't show up in gc.get_referrers()
+    label.append(safe_repr(obj))
+    if extra_info:
+        label.append(str(extra_info(obj)))
+    return quote('\n'.join(label))
 
 
 def quote(s):
@@ -426,14 +554,22 @@ def short_repr(obj):
                         types.BuiltinFunctionType)):
         return obj.__name__
     if isinstance(obj, types.MethodType):
-        if obj.im_self is not None:
-            return obj.im_func.__name__ + ' (bound)'
-        else:
-            return obj.im_func.__name__
+        try:
+            if obj.__self__ is not None:
+                return obj.__func__.__name__ + ' (bound)'
+            else:
+                return obj.__func__.__name__
+        except AttributeError:
+            # Python < 2.6 compatibility
+            if obj.im_self is not None:
+                return obj.im_func.__name__ + ' (bound)'
+            else:
+                return obj.im_func.__name__
+
+    if isinstance(obj, types.FrameType):
+        return '%s:%s' % (obj.f_code.co_filename, obj.f_lineno)
     if isinstance(obj, (tuple, list, dict, set)):
         return '%d items' % len(obj)
-    if isinstance(obj, weakref.ref):
-        return 'all_weakrefs_are_one'
     return repr(obj)[:40]
 
 
@@ -453,8 +589,25 @@ def gradient(start_color, end_color, depth, max_depth):
 def edge_label(source, target):
     if isinstance(target, dict) and target is getattr(source, '__dict__', None):
         return ' [label="__dict__",weight=10]'
-    elif isinstance(source, dict):
-        for k, v in source.iteritems():
+    if isinstance(source, types.FrameType):
+        if target is source.f_locals:
+            return ' [label="f_locals",weight=10]'
+        if target is source.f_globals:
+            return ' [label="f_globals",weight=10]'
+    if isinstance(source, types.MethodType):
+        try:
+            if target is source.__self__:
+                return ' [label="__self__",weight=10]'
+            if target is source.__func__:
+                return ' [label="__func__",weight=10]'
+        except AttributeError:
+            # Python < 2.6 compatibility
+            if target is source.im_self:
+                return ' [label="im_self",weight=10]'
+            if target is source.im_func:
+                return ' [label="im_func",weight=10]'
+    if isinstance(source, dict):
+        for k, v in iteritems(source):
             if v is target:
                 if isinstance(k, basestring) and k:
                     return ' [label="%s",weight=2]' % quote(k)
